@@ -32,6 +32,7 @@
 // --- DABlinTextOptions -----------------------------------------------------------------
 struct DABlinTextOptions {
 	std::string filename;
+	std::string initial_label;
 	int initial_sid;
 	int initial_scids;
 	int initial_subchid_dab;
@@ -40,6 +41,7 @@ struct DABlinTextOptions {
 	std::string dab_live_source_type;
 	std::string initial_channel;
 	bool pcm_output;
+	bool untouched_output;
 	int gain;
 DABlinTextOptions() :
 	initial_sid(LISTED_SERVICE::sid_none),
@@ -48,6 +50,7 @@ DABlinTextOptions() :
 	initial_subchid_dab_plus(AUDIO_SERVICE::subchid_none),
 	dab_live_source_type(DABLiveETISource::TYPE_DAB2ETI),
 	pcm_output(false),
+	untouched_output(false),
 	gain(DAB_LIVE_SOURCE_CHANNEL::auto_gain)
 	{}
 };
@@ -62,7 +65,7 @@ private:
 	ETIPlayer *eti_player;
 	FICDecoder *fic_decoder;
 
-	void ETIProcessFrame(const uint8_t *data) {eti_player->ProcessFrame(data);};
+	void ETIProcessFrame(const uint8_t *data) {eti_player->ProcessFrame(data);}
 	void ETIUpdateProgress(const ETI_PROGRESS progress);
 	void ETIProcessFIC(const uint8_t *data, size_t len) {fic_decoder->Process(data, len);}
 	void FICChangeService(const LISTED_SERVICE& service);
