@@ -28,6 +28,8 @@ static void break_handler(int) {
 
 
 static void usage(const char* exe) {
+	DABlinGTKOptions options_default;
+
 	fprint_dablin_banner(stderr);
 	fprintf(stderr, "Usage: %s [OPTIONS] [file]\n", exe);
 	fprintf(stderr, "  -h           Show this help\n"
@@ -36,15 +38,19 @@ static void usage(const char* exe) {
 					"  -C <ch>,...  Channels to be listed (comma separated; requires DAB live source;\n"
 					"               an optional gain can also be specified, e.g. \"5C:-54\")\n"
 					"  -c <ch>      Channel to be played (requires DAB live source)\n"
+					"  -l <label>   Label of the service to be played\n"
 					"  -s <sid>     ID of the service to be played\n"
 					"  -x <scids>   ID of the service component to be played (requires service ID)\n"
 					"  -g <gain>    USB stick gain to pass to DAB live source (auto gain is default)\n"
+					"  -r <path>    Path for recordings (default: %s)\n"
 					"  -p           Output PCM to stdout instead of using SDL\n"
+					"  -u           Output untouched audio stream to stdout instead of using SDL\n"
 					"  -S           Initially disable slideshow\n"
 					"  -L           Enable loose behaviour (e.g. PAD conformance)\n"
 					"  file         Input file to be played (stdin, if not specified)\n",
 					DABLiveETISource::TYPE_DAB2ETI.c_str(),
-					DABLiveETISource::TYPE_ETI_CMDLINE.c_str()
+					DABLiveETISource::TYPE_ETI_CMDLINE.c_str(),
+					options_default.recordings_path.c_str()
 			);
 	exit(1);
 }
@@ -62,10 +68,11 @@ int main(int argc, char **argv) {
 	}
 
 	DABlinGTKOptions options;
+	int initial_param_count = 0;
 
 	// option args
 	int c;
-	while((c = getopt(argc, argv, "hd:D:C:c:g:s:x:pSL")) != -1) {
+	while((c = getopt(argc, argv, "hd:D:C:c:l:g:r:s:x:puSL")) != -1) {
 		switch(c) {
 		case 'h':
 			usage(argv[0]);
@@ -82,8 +89,13 @@ int main(int argc, char **argv) {
 		case 'c':
 			options.initial_channel = optarg;
 			break;
+		case 'l':
+			options.initial_label = optarg;
+			initial_param_count++;
+			break;
 		case 's':
 			options.initial_sid = strtol(optarg, nullptr, 0);
+			initial_param_count++;
 			break;
 		case 'x':
 			options.initial_scids = strtol(optarg, nullptr, 0);
@@ -91,9 +103,15 @@ int main(int argc, char **argv) {
 		case 'g':
 			options.gain = strtol(optarg, nullptr, 0);
 			break;
+		case 'r':
+			options.recordings_path = optarg;
+			break;
 		case 'p':
 			options.pcm_output = true;
 			break;
+		case 'u':
+			options.untouched_output = true;
+			break;
 		case 'S':
 			options.initially_disable_slideshow = true;
 			break;
@@ -152,6 +170,18 @@ int main(int argc, char **argv) {
 	}
 #endif
 
+	if(options.pcm_output && options.untouched_output) {
+		fprintf(stderr, "No more than one output option can be specified!\n");
+		usage(argv[0]);
+	}
+
+
+	// at most one initial param needed!
+	if(initial_param_count > 1) {
+		fprintf(stderr, "At most one initial parameter shall be specified!\n");
+		usage(argv[0]);
+	}
+
 
 	fprint_dablin_banner(stderr);
 
@@ -172,6 +202,7 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 	this->options = options;
 
 	initial_channel_appended = false;
+	rec_file = nullptr;
 
 	slideshow_window.set_transient_for(*this);
 
@@ -182,7 +213,7 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 	pad_change_dynamic_label.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeDynamicLabelEmitted));
 	pad_change_slide.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeSlideEmitted));
 
-	eti_player = new ETIPlayer(options.pcm_output, this);
+	eti_player = new ETIPlayer(options.pcm_output, options.untouched_output, this);
 
 	if(!options.dab_live_source_binary.empty()) {
 		eti_source = nullptr;
@@ -250,6 +281,8 @@ int DABlinGTK::ComboServicesSlotCompare(const Gtk::TreeModel::iterator& a, const
 }
 
 void DABlinGTK::InitWidgets() {
+	this->signal_delete_event().connect(sigc::mem_fun(*this, &DABlinGTK::on_window_delete_event));
+
 	// init widgets
 	frame_combo_channels.set_label("Channel");
 	frame_combo_channels.set_size_request(75, -1);
@@ -295,6 +328,10 @@ void DABlinGTK::InitWidgets() {
 	label_format.set_halign(Gtk::ALIGN_START);
 	label_format.set_padding(WIDGET_SPACE, WIDGET_SPACE);
 
+	tglbtn_record.set_image_from_icon_name("media-record");
+	tglbtn_record.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_tglbtn_record));
+	tglbtn_record.set_sensitive(false);
+
 	tglbtn_mute.set_label("Mute");
 	tglbtn_mute.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_tglbtn_mute));
 
@@ -330,11 +367,12 @@ void DABlinGTK::InitWidgets() {
 	top_grid.attach_next_to(frame_label_ensemble, frame_combo_channels, Gtk::POS_RIGHT, 1, 2);
 	top_grid.attach_next_to(frame_combo_services, frame_label_ensemble, Gtk::POS_RIGHT, 1, 2);
 	top_grid.attach_next_to(frame_label_format, frame_combo_services, Gtk::POS_RIGHT, 1, 2);
-	top_grid.attach_next_to(tglbtn_mute, frame_label_format, Gtk::POS_RIGHT, 1, 1);
+	top_grid.attach_next_to(tglbtn_record, frame_label_format, Gtk::POS_RIGHT, 1, 1);
+	top_grid.attach_next_to(tglbtn_mute, tglbtn_record, Gtk::POS_RIGHT, 1, 1);
 	top_grid.attach_next_to(vlmbtn, tglbtn_mute, Gtk::POS_RIGHT, 1, 1);
-	top_grid.attach_next_to(tglbtn_slideshow, tglbtn_mute, Gtk::POS_BOTTOM, 2, 1);
-	top_grid.attach_next_to(frame_label_dl, frame_combo_channels, Gtk::POS_BOTTOM, 6, 1);
-	top_grid.attach_next_to(progress_position, frame_label_dl, Gtk::POS_BOTTOM, 6, 1);
+	top_grid.attach_next_to(tglbtn_slideshow, tglbtn_record, Gtk::POS_BOTTOM, 3, 1);
+	top_grid.attach_next_to(frame_label_dl, frame_combo_channels, Gtk::POS_BOTTOM, 7, 1);
+	top_grid.attach_next_to(progress_position, frame_label_dl, Gtk::POS_BOTTOM, 7, 1);
 
 	show_all_children();
 	progress_position.hide();	// invisible until progress updated
@@ -381,7 +419,7 @@ void DABlinGTK::AddChannel(dab_channels_t::const_iterator &it, int gain) {
 }
 
 void DABlinGTK::SetService(const LISTED_SERVICE& service) {
-	// (re)set labels/tooltips
+	// (re)set labels/tooltips/record button
 	if(!service.IsNone()) {
 		char sid_string[7];
 		snprintf(sid_string, sizeof(sid_string), "0x%04X", service.sid);
@@ -410,10 +448,12 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 			}
 			frame_label_format.set_tooltip_text(tooltip_text);
 		}
+		tglbtn_record.set_sensitive(true);
 	} else {
 		set_title("DABlin");
 		frame_combo_services.set_tooltip_text("");
 		frame_label_format.set_tooltip_text("");
+		tglbtn_record.set_sensitive(false);
 	}
 
 	// if the audio service changed, reset format/DL/slide + switch
@@ -447,6 +487,86 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 }
 
 
+void DABlinGTK::on_tglbtn_record() {
+	if(tglbtn_record.get_active()) {
+		// get timestamp
+		const time_t now = time(nullptr);
+		if(now == (time_t) -1)
+			perror("DABlinGTK: error while getting time");
+		struct tm* now_tm = localtime(&now);
+		if(!now_tm)
+			perror("DABlinGTK: error while getting local time");
+		char now_string[22];
+		strftime(now_string, sizeof(now_string), "%F - %H-%M-%S", now_tm);
+
+		// get station name
+		LISTED_SERVICE service;	// default: none
+		Gtk::TreeModel::iterator row_it = combo_services.get_active();
+		if(combo_services_liststore->iter_is_valid(row_it)) {
+			Gtk::TreeModel::Row row = *row_it;
+			service = row[combo_services_cols.col_service];
+		}
+		std::string label = FICDecoder::ConvertLabelToUTF8(service.label);
+
+		// escape forbidden '/' character
+		std::string label_cleaned = label;
+		for(size_t i = 0; i < label_cleaned.length(); i++)
+			if(label_cleaned[i] == '/')
+				label_cleaned[i] = '_';
+
+		std::string rec_filename = options.recordings_path + "/" + std::string(now_string) + " - " + label_cleaned + "." + eti_player->GetUntouchedStreamFileExtension();
+		FILE* new_rec_file = fopen(rec_filename.c_str(), "wb");
+		if(new_rec_file) {
+			// disable channel/service switch
+			combo_channels.set_sensitive(false);	// parent frame already non-sensitive, if channels not available
+			combo_services.set_sensitive(false);
+
+			fprintf(stderr, "DABlinGTK: recording started into '%s'\n", rec_filename.c_str());
+
+			{
+				std::lock_guard<std::mutex> lock(rec_file_mutex);
+				rec_file = new_rec_file;
+			}
+
+			eti_player->AddUntouchedStreamConsumer(this);
+		} else {
+			int fopen_errno = errno;	// save before overwrite
+			tglbtn_record.set_active(false);
+
+			Gtk::MessageDialog hint(*this, "Could not start recording!", false, Gtk::MESSAGE_ERROR);
+			hint.set_title("DABlinGTK");
+			hint.set_secondary_text(strerror(fopen_errno), false);
+			hint.run();
+		}
+	} else {
+		std::lock_guard<std::mutex> lock(rec_file_mutex);
+
+		// only stop recording, if active (handles error on recording start)
+		if(rec_file) {
+			eti_player->RemoveUntouchedStreamConsumer(this);
+
+			fclose(rec_file);
+			rec_file = nullptr;
+
+			fprintf(stderr, "DABlinGTK: recording stopped\n");
+
+			// enable channel/service switch
+			combo_channels.set_sensitive(true);		// parent frame already non-sensitive, if channels not available
+			combo_services.set_sensitive(true);
+		}
+	}
+}
+
+void DABlinGTK::ProcessUntouchedStream(const uint8_t* data, size_t len) {
+	std::lock_guard<std::mutex> lock(rec_file_mutex);
+
+	if(rec_file) {
+		if(fwrite(data, len, 1, rec_file) != 1)
+			perror("DABlinGTK: error while writing untouched stream to file");
+	}
+}
+
+
 void DABlinGTK::on_tglbtn_mute() {
 	eti_player->SetAudioMute(tglbtn_mute.get_active());
 }
@@ -466,6 +586,19 @@ void DABlinGTK::on_tglbtn_slideshow() {
 		slideshow_window.hide();
 }
 
+bool DABlinGTK::on_window_delete_event(GdkEventAny* /*any_event*/) {
+	// prevent exit while recording
+	if(tglbtn_record.get_active()) {
+		Gtk::MessageDialog hint(*this, "Cannot exit while recording!", false, Gtk::MESSAGE_ERROR);
+		hint.set_title("DABlinGTK");
+		hint.set_secondary_text("The recording has to be stopped first.", false);
+		hint.run();
+		return true;
+	} else {
+		return false;
+	}
+}
+
 void DABlinGTK::ConnectKeyPressEventHandler(Gtk::Widget& widget) {
 	widget.signal_key_press_event().connect(sigc::mem_fun(*this, &DABlinGTK::HandleKeyPressEvent));
 	widget.add_events(Gdk::KEY_PRESS_MASK);
@@ -480,11 +613,72 @@ bool DABlinGTK::HandleKeyPressEvent(GdkEventKey* key_event) {
 			// toggle mute
 			tglbtn_mute.clicked();
 			return true;
+		case GDK_KEY_r:
+		case GDK_KEY_R:
+			// toggle record, if allowed
+			if(tglbtn_record.get_sensitive())
+				tglbtn_record.clicked();
+			return true;
+		// try to switch service
+		case GDK_KEY_1:
+		case GDK_KEY_KP_1:
+			TryServiceSwitch(0);
+			return true;
+		case GDK_KEY_2:
+		case GDK_KEY_KP_2:
+			TryServiceSwitch(1);
+			return true;
+		case GDK_KEY_3:
+		case GDK_KEY_KP_3:
+			TryServiceSwitch(2);
+			return true;
+		case GDK_KEY_4:
+		case GDK_KEY_KP_4:
+			TryServiceSwitch(3);
+			return true;
+		case GDK_KEY_5:
+		case GDK_KEY_KP_5:
+			TryServiceSwitch(4);
+			return true;
+		case GDK_KEY_6:
+		case GDK_KEY_KP_6:
+			TryServiceSwitch(5);
+			return true;
+		case GDK_KEY_7:
+		case GDK_KEY_KP_7:
+			TryServiceSwitch(6);
+			return true;
+		case GDK_KEY_8:
+		case GDK_KEY_KP_8:
+			TryServiceSwitch(7);
+			return true;
+		case GDK_KEY_9:
+		case GDK_KEY_KP_9:
+			TryServiceSwitch(8);
+			return true;
+		case GDK_KEY_0:
+		case GDK_KEY_KP_0:
+			TryServiceSwitch(9);
+			return true;
+		case GDK_KEY_minus:
+		case GDK_KEY_KP_Subtract:
+			TryServiceSwitch(combo_services.get_active_row_number() - 1);
+			return true;
+		case GDK_KEY_plus:
+		case GDK_KEY_KP_Add:
+			TryServiceSwitch(combo_services.get_active_row_number() + 1);
+			return true;
 		}
 	}
 	return false;
 }
 
+void DABlinGTK::TryServiceSwitch(int index) {
+	// switch service, if allowed and index valid
+	if(combo_services.is_sensitive() && index >= 0 && index < (signed) combo_services_liststore->children().size())
+		combo_services.set_active(index);
+}
+
 bool DABlinGTK::HandleConfigureEvent(GdkEventConfigure* /*configure_event*/) {
 	// move together with slideshow window
 	if(slideshow_window.get_visible())
@@ -529,9 +723,10 @@ void DABlinGTK::FICChangeServiceEmitted() {
 
 	LISTED_SERVICE new_service = fic_change_service.Pop();
 
-	Glib::ustring label = FICDecoder::ConvertLabelToUTF8(new_service.label);
+	std::string label = FICDecoder::ConvertLabelToUTF8(new_service.label);
+	Glib::ustring combo_label = label;
 	if(new_service.multi_comps)
-		label = (!new_service.IsPrimary() ? "» " : "") + label + (new_service.IsPrimary() ? " »" : "");
+		combo_label = (!new_service.IsPrimary() ? "» " : "") + combo_label + (new_service.IsPrimary() ? " »" : "");
 
 	// get row (add new one, if needed)
 	Gtk::ListStore::Children children = combo_services_liststore->children();
@@ -547,12 +742,12 @@ void DABlinGTK::FICChangeServiceEmitted() {
 		row_it = combo_services_liststore->append();
 
 	Gtk::TreeModel::Row row = *row_it;
-	row[combo_services_cols.col_string] = label;
+	row[combo_services_cols.col_string] = combo_label;
 	row[combo_services_cols.col_service] = new_service;
 
 	if(add_new_row) {
 		// set (initial) service
-		if(new_service.sid == options.initial_sid && new_service.scids == options.initial_scids)
+		if(label == options.initial_label || (new_service.sid == options.initial_sid && new_service.scids == options.initial_scids))
 			combo_services.set_active(row_it);
 	} else {
 		// set (updated) service
@@ -582,8 +777,9 @@ void DABlinGTK::on_combo_channels() {
 			"Gain: " + channel.GainToString()
 	);
 
-	// prevent re-use of initial SID
+	// prevent re-use of initial params
 	if(initial_channel_appended) {
+		options.initial_label = "";
 		options.initial_sid = LISTED_SERVICE::sid_none;
 		options.initial_scids = LISTED_SERVICE::scids_none;
 	}
@@ -637,134 +833,17 @@ void DABlinGTK::PADChangeSlideEmitted() {
 		slideshow_window.TryToShow();
 }
 
+void DABlinGTK::PADLengthError(size_t /*announced_xpad_len*/, size_t /*xpad_len*/) {
+	fprintf(stderr, "\x1B[31m" "[X-PAD len]" "\x1B[0m" " ");
+}
+
 
 Glib::ustring DABlinGTK::DeriveShortLabel(Glib::ustring long_label, uint16_t short_label_mask) {
 	Glib::ustring short_label;
 
-	for(int i = 0; i < 16; i++)
+	for(size_t i = 0; i < long_label.length(); i++)		// consider discarded trailing spaces
 		if(short_label_mask & (0x8000 >> i))
 			short_label += long_label[i];
 
 	return short_label;
 }
-
-
-// --- DABlinGTKSlideshowWindow -----------------------------------------------------------------
-DABlinGTKSlideshowWindow::DABlinGTKSlideshowWindow() {
-	pixbuf_waiting = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, 320, 240);	// default slide size
-	pixbuf_waiting->fill(0x003000FF);
-
-	// align to the right of parent
-	offset_x = 20;	// add some horizontal padding for WM decoration
-	offset_y = 0;
-
-	set_type_hint(Gdk::WINDOW_TYPE_HINT_UTILITY);
-	set_resizable(false);
-	set_deletable(false);
-
-	add(top_grid);
-
-	top_grid.attach(image, 0, 0, 1, 1);
-	top_grid.attach_next_to(link_button, image, Gtk::POS_BOTTOM, 1, 1);
-
-	show_all_children();
-
-	// add window config event handler (before default handler)
-	signal_configure_event().connect(sigc::mem_fun(*this, &DABlinGTKSlideshowWindow::HandleConfigureEvent), false);
-	add_events(Gdk::STRUCTURE_MASK);
-}
-
-void DABlinGTKSlideshowWindow::TryToShow() {
-	// if already visible or no slide (awaiting), abort
-	if(get_visible() || image.get_storage_type() == Gtk::ImageType::IMAGE_EMPTY)
-		return;
-
-	AlignToParent();
-	show();
-}
-
-void DABlinGTKSlideshowWindow::AlignToParent() {
-	int x, y, w, h;
-	get_transient_for()->get_position(x, y);
-	get_transient_for()->get_size(w, h);
-
-	// TODO: fix multi head issue
-	move(x + w + offset_x, y + offset_y);
-}
-
-bool DABlinGTKSlideshowWindow::HandleConfigureEvent(GdkEventConfigure* /*configure_event*/) {
-	// update window offset, if visible
-	if(get_visible()) {
-		int x, y, w, h, sls_x, sls_y;
-		get_transient_for()->get_position(x, y);
-		get_transient_for()->get_size(w, h);
-		get_position(sls_x, sls_y);	// event position doesn't work!
-
-		offset_x = sls_x - w - x;
-		offset_y = sls_y - y;
-	}
-	return false;
-}
-
-void DABlinGTKSlideshowWindow::AwaitSlide() {
-	set_title("Slideshow...");
-
-	image.set(pixbuf_waiting);
-	image.set_tooltip_text("Waiting for slide...");
-
-	link_button.hide();
-}
-
-void DABlinGTKSlideshowWindow::UpdateSlide(const MOT_FILE& slide) {
-	std::string type_mime = "";
-	std::string type_display = "unknown";
-
-	switch(slide.content_sub_type) {
-	case MOT_FILE::CONTENT_SUB_TYPE_JFIF:
-		type_mime = "image/jpeg";
-		type_display = "JPEG";
-		break;
-	case MOT_FILE::CONTENT_SUB_TYPE_PNG:
-		type_mime = "image/png";
-		type_display = "PNG";
-		break;
-	}
-
-	Glib::RefPtr<Gdk::PixbufLoader> pixbuf_loader = Gdk::PixbufLoader::create(type_mime, true);
-	pixbuf_loader->write(&slide.data[0], slide.data.size());
-	pixbuf_loader->close();
-
-	Glib::RefPtr<Gdk::Pixbuf> pixbuf = pixbuf_loader->get_pixbuf();
-	if(!pixbuf)
-		return;
-
-	// update title
-	std::string title = "Slideshow";
-	if(!slide.category_title.empty())
-		title = slide.category_title + " - " + title;
-	set_title(title);
-
-	// update image
-	image.set(pixbuf);
-	image.set_tooltip_text(
-			"Resolution: " + std::to_string(pixbuf->get_width()) + "x" + std::to_string(pixbuf->get_height()) + " pixels\n" +
-			"Size: " + std::to_string(slide.data.size()) + " bytes\n" +
-			"Format: " + type_display + "\n" +
-			"Content name: " + slide.content_name);
-
-	// update ClickThroughURL link
-	if(!slide.click_through_url.empty()) {
-		link_button.set_label(slide.click_through_url);
-		link_button.set_tooltip_text(slide.click_through_url);
-		link_button.set_uri(slide.click_through_url);
-
-		// ensure that the label (created/replaced by set_label) does not extend the slide
-		Gtk::Label* l = (Gtk::Label*) link_button.get_child();
-		l->set_max_width_chars(1);
-		l->set_ellipsize(Pango::ELLIPSIZE_END);
-
-		link_button.show();
-	} else {
-		link_button.hide();
-	}
-}
