@@ -25,9 +25,11 @@
 #include <signal.h>
 #include <string>
 #include <thread>
+#include <time.h>
 
 #include <gtkmm.h>
 
+#include "dablin_gtk_sls.h"
 #include "eti_source.h"
 #include "eti_player.h"
 #include "fic_decoder.h"
@@ -39,30 +41,6 @@
 
 
 
-// --- DABlinGTKSlideshowWindow -----------------------------------------------------------------
-class DABlinGTKSlideshowWindow : public Gtk::Window {
-private:
-	Gtk::Grid top_grid;
-	Gtk::Image image;
-	Gtk::LinkButton link_button;
-
-	Glib::RefPtr<Gdk::Pixbuf> pixbuf_waiting;
-	std::atomic<int> offset_x;
-	std::atomic<int> offset_y;
-
-	bool HandleConfigureEvent(GdkEventConfigure* configure_event);
-public:
-	DABlinGTKSlideshowWindow();
-
-	void TryToShow();
-	void AlignToParent();
-
-	void AwaitSlide();
-	void UpdateSlide(const MOT_FILE& slide);
-	void ClearSlide() {image.clear();}
-};
-
-
 // --- DABlinGTKChannelColumns -----------------------------------------------------------------
 class DABlinGTKChannelColumns : public Gtk::TreeModelColumnRecord {
 public:
@@ -92,13 +70,16 @@ public:
 // --- DABlinGTKOptions -----------------------------------------------------------------
 struct DABlinGTKOptions {
 	std::string filename;
+	std::string initial_label;
 	int initial_sid;
 	int initial_scids;
 	std::string dab_live_source_binary;
 	std::string dab_live_source_type;
 	std::string displayed_channels;
 	std::string initial_channel;
+	std::string recordings_path;
 	bool pcm_output;
+	bool untouched_output;
 	int gain;
 	bool initially_disable_slideshow;
 	bool loose;
@@ -107,7 +88,9 @@ DABlinGTKOptions() :
 	initial_sid(LISTED_SERVICE::sid_none),
 	initial_scids(LISTED_SERVICE::scids_none),
 	dab_live_source_type(DABLiveETISource::TYPE_DAB2ETI),
+	recordings_path("/tmp"),
 	pcm_output(false),
+	untouched_output(false),
 	gain(DAB_LIVE_SOURCE_CHANNEL::auto_gain),
 	initially_disable_slideshow(false),
 	loose(false)
@@ -144,7 +127,7 @@ public:
 
 
 // --- DABlinGTK -----------------------------------------------------------------
-class DABlinGTK : public Gtk::Window, ETISourceObserver, ETIPlayerObserver, FICDecoderObserver, PADDecoderObserver {
+class DABlinGTK : public Gtk::Window, ETISourceObserver, ETIPlayerObserver, FICDecoderObserver, PADDecoderObserver, UntouchedStreamConsumer {
 private:
 	DABlinGTKOptions options;
 
@@ -161,10 +144,13 @@ private:
 	FICDecoder *fic_decoder;
 	PADDecoder *pad_decoder;
 
+	std::mutex rec_file_mutex;
+	FILE* rec_file;
+
 	// ETI data change
 	GTKDispatcherQueue<ETI_PROGRESS> eti_update_progress;
-	void ETIProcessFrame(const uint8_t *data) {eti_player->ProcessFrame(data);};
-	void ETIUpdateProgress(const ETI_PROGRESS progress) {eti_update_progress.PushAndEmit(progress);};
+	void ETIProcessFrame(const uint8_t *data) {eti_player->ProcessFrame(data);}
+	void ETIUpdateProgress(const ETI_PROGRESS progress) {eti_update_progress.PushAndEmit(progress);}
 	void ETIUpdateProgressEmitted();
 
 	GTKDispatcherQueue<std::string> eti_change_format;
@@ -172,9 +158,11 @@ private:
 	void ETIChangeFormatEmitted();
 
 	void ETIProcessFIC(const uint8_t *data, size_t len) {fic_decoder->Process(data, len);}
-	void ETIResetFIC() {fic_decoder->Reset();};
+	void ETIResetFIC() {fic_decoder->Reset();}
 	void ETIProcessPAD(const uint8_t *xpad_data, size_t xpad_len, bool exact_xpad_len, const uint8_t* fpad_data) {pad_decoder->Process(xpad_data, xpad_len, exact_xpad_len, fpad_data);}
 
+	void ProcessUntouchedStream(const uint8_t* data, size_t len);
+
 
 	Gtk::Grid top_grid;
 
@@ -196,6 +184,7 @@ private:
 	Gtk::Frame frame_label_format;
 	Gtk::Label label_format;
 
+	Gtk::ToggleButton tglbtn_record;
 	Gtk::ToggleButton tglbtn_mute;
 	Gtk::VolumeButton vlmbtn;
 	Gtk::ToggleButton tglbtn_slideshow;
@@ -212,15 +201,18 @@ private:
 
 	void SetService(const LISTED_SERVICE& service);
 
+	void on_tglbtn_record();
 	void on_tglbtn_mute();
 	void on_vlmbtn(double value);
 	void on_tglbtn_slideshow();
 	void on_combo_channels();
 	void on_combo_services();
+	bool on_window_delete_event(GdkEventAny* any_event);
 
 	void ConnectKeyPressEventHandler(Gtk::Widget& widget);
 	bool HandleKeyPressEvent(GdkEventKey* key_event);
 	bool HandleConfigureEvent(GdkEventConfigure* configure_event);
+	void TryServiceSwitch(int index);
 
 	// FIC data change
 	GTKDispatcherQueue<FIC_ENSEMBLE> fic_change_ensemble;
@@ -240,6 +232,8 @@ private:
 	void PADChangeSlide(const MOT_FILE& slide) {pad_change_slide.PushAndEmit(slide);}
 	void PADChangeSlideEmitted();
 
+	void PADLengthError(size_t announced_xpad_len, size_t xpad_len);
+
 	Glib::ustring DeriveShortLabel(Glib::ustring long_label, uint16_t short_label_mask);
 public:
 	DABlinGTK(DABlinGTKOptions options);
