@@ -408,7 +408,7 @@ void FICDecoder::ProcessFIG1_0(uint16_t eid, const FIC_LABEL& label) {
 		ensemble.label = label;
 
 		std::string label_str = ConvertLabelToUTF8(label);
-		fprintf(stderr, "FICDecoder: EId 0x%04X: ensemble label '%s'\n", eid, label_str.c_str());
+		fprintf(stderr, "FICDecoder: EId 0x%04X: ensemble label '" "\x1B[32m" "%s" "\x1B[0m" "'\n", eid, label_str.c_str());
 
 		observer->FICChangeEnsemble(ensemble);
 	}
@@ -420,7 +420,7 @@ void FICDecoder::ProcessFIG1_1(uint16_t sid, const FIC_LABEL& label) {
 		service.label = label;
 
 		std::string label_str = ConvertLabelToUTF8(label);
-		fprintf(stderr, "FICDecoder: SId 0x%04X: programme service label '%s'\n", sid, label_str.c_str());
+		fprintf(stderr, "FICDecoder: SId 0x%04X: programme service label '" "\x1B[32m" "%s" "\x1B[0m" "'\n", sid, label_str.c_str());
 
 		UpdateService(service);
 	}
@@ -435,7 +435,7 @@ void FICDecoder::ProcessFIG1_4(uint16_t sid, int scids, const FIC_LABEL& label)
 		comp_label = label;
 
 		std::string label_str = ConvertLabelToUTF8(label);
-		fprintf(stderr, "FICDecoder: SId 0x%04X, SCIdS %2d: service component label '%s'\n", sid, scids, label_str.c_str());
+		fprintf(stderr, "FICDecoder: SId 0x%04X, SCIdS %2d: service component label '" "\x1B[32m" "%s" "\x1B[0m" "'\n", sid, scids, label_str.c_str());
 
 		UpdateService(service);
 	}
